<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;
use Laravelista\Algo\Helpers\Slug;
use Laravelista\Algo\Models\Category;
use Laravelista\Picasso\Picasso;

class CategoryController extends Controller
{
    public function __construct(
        protected ResponseFactory $response
    ) {
    }

    /**
     * Display a listing of the resource.
     */
    public function index(): Response
    {
        $categories = Category::with(['translations'])->get()->sortBy('name');

        return $this->response->view(
            view: 'algo::categories.index',
            data: ['categories' => $categories]
        );
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(): Response
    {
        return $this->response->view(view: 'algo::categories.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request, Slug $slug, Picasso $picasso): RedirectResponse
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('category_translations')
                    ->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $category = new Category();
        $category->translateOrNew($locale)->name = $validatedData['name'];
        $category->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['name']);
        $category->translateOrNew($locale)->description = $validatedData['description'] ?? null;
        if ($request->hasFile('image')) {
            $category->setAttribute('image', $request->image->store('images', 'originals'));
        }
        $category->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($category->getAttribute('image'), Config::get(key: 'algo.category_dimensions'), 'public');
        }

        return $this->response
            ->redirectToRoute(
                route: 'algo.categories.index'
            )
            ->with('status', 'Kategorija napravljena!');
    }

    /**
     * Display the specified resource.
     */
    public function show(Category $category): Response
    {
        return $this->response->view(
            view: 'algo::categories.show',
            data: ['category' => $category]
        );
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Category $category): Response
    {
        return $this->response->view(
            view: 'algo::categories.edit',
            data: ['category' => $category]
        );
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Category $category, Slug $slug, Storage $storage, Picasso $picasso): RedirectResponse
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('category_translations')
                    ->ignore($category->getAttribute('id'), 'category_id')
                    ->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $category->translateOrNew($locale)->name = $validatedData['name'];
        $category->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['name']);
        $category->translateOrNew($locale)->description = $validatedData['description'] ?? null;

        if ($request->hasFile('image') and $category->hasImage()) {
            $storage::disk('originals')->delete($category->getAttribute('image'));
            $picasso->purge($category->getAttribute('image'), 'public');
        }

        if ($request->hasFile('image')) {
            $category->setAttribute('image', $request->image->store('images', 'originals'));
        }

        $category->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($category->getAttribute('image'), Config::get(key: 'algo.category_dimensions'), 'public');
        }

        return $this->response
            ->redirectToRoute(
                route: 'algo.categories.index'
            )
            ->with('status', 'Kategorija uređena!');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Category $category, Storage $storage, Picasso $picasso): RedirectResponse
    {
        if ($category->hasImage()) {
            $storage::disk('originals')->delete($category->getAttribute('image'));
            $picasso->purge($category->getAttribute('image'), 'public');
        }

        $category->delete();

        return $this->response
            ->redirectToRoute(
                route: 'algo.categories.index'
            )
            ->with('status', 'Kategorija obrisana!');
    }
}
