<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;
use Laravelista\Algo\Helpers\PublishedAt;
use Laravelista\Algo\Helpers\Slug;
use Laravelista\Algo\Models\Category;
use Laravelista\Algo\Models\Post;
use Laravelista\Algo\Models\Tag;
use Laravelista\Picasso\Picasso;

class PostController extends Controller
{
    public function __construct(
        protected ResponseFactory $response
    ) {
    }

    /**
     * Display a listing of the resource.
     */
    public function index(): Response
    {
        $posts = Post::
            with(['translations', 'tags.translations', 'category.translations', 'author'])
            ->orderByRaw('-published_at ASC')
            ->orderBy('created_at', 'DESC')
            ->get();

        return $this->response->view(
            view: 'algo::posts.index',
            data: [
                'posts' => $posts
            ]
        );
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(): Response
    {
        return $this->response->view(
            view: 'algo::posts.create',
            data: [
                'authors' => Config::get('algo.user')::pluck('name', 'id')->toArray(),
                'tags' => Tag::with(['translations'])->get()->pluck('name', 'id')->toArray(),
                'categories' => Category::with(['translations'])->get()->pluck('name', 'id')->toArray()
            ]
        );
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request, PublishedAt $publishedAt, Slug $slug): RedirectResponse
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'title' => ['required', 'string', 'max:255'],
            'intro' => ['nullable', 'string', 'max:255'],
            'content' => ['required', 'string'],
            'video_link' => ['nullable', 'url', 'max:255'],
            'published_at_date' => ['nullable', 'date'],
            'published_at_time' => ['nullable', 'date_format:H:i'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('post_translations')
                    ->where('locale', $locale)
            ],
            'author_id' => ['required', 'integer', 'min:1', 'exists:users,id'],
            'category_id' => ['required', 'integer', 'min:1', 'exists:categories,id'],
            'tags_list' => ['nullable', 'array'],
            'tags_list.*' => ['required', 'integer', 'min:1', 'exists:tags,id'],
            'featured' => ['nullable', 'boolean']
        ]);

        $post = new Post();
        $post->setAttribute('featured', $validatedData['featured'] ?? false);
        $post->setAttribute('video_link', $validatedData['video_link'] ?? null);
        $post->translateOrNew($locale)->title = $validatedData['title'];
        $post->translateOrNew($locale)->intro = $validatedData['intro'] ?? null;
        $post->translateOrNew($locale)->content = $validatedData['content'];
        $post->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['title']);
        $post->author()->associate($validatedData['author_id']);
        $post->category()->associate($validatedData['category_id']);
        $post->setAttribute('published_at', $publishedAt->get($validatedData['published_at_date'], $validatedData['published_at_time']));
        $post->save();
        $post->tags()->sync($validatedData['tags_list'] ?? []);

        return $this->response
            ->redirectToRoute('algo.posts.index')
            ->with('status', 'Objava napravljena!');
    }

    /**
     * Display the specified resource.
     */
    public function show(Post $post): Response
    {
        $post->load(['tags.translations']);

        return $this->response->view(
            view: 'algo::posts.show',
            data: ['post' => $post]
        );
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Post $post): Response
    {
        return $this->response->view(
            view: 'algo::posts.edit',
            data: [
                'post' => $post->load(['translations', 'tags', 'category', 'author']),
                'authors' => Config::get('algo.user')::pluck('name', 'id')->toArray(),
                'tags' => Tag::with(['translations'])->get()->pluck('name', 'id')->toArray(),
                'categories' => Category::with(['translations'])->get()->pluck('name', 'id')->toArray()
            ]
        );
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Post $post, PublishedAt $publishedAt, Slug $slug): RedirectResponse
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'title' => ['required', 'string', 'max:255'],
            'intro' => ['nullable', 'string', 'max:255'],
            'content' => ['required', 'string'],
            'video_link' => ['nullable', 'url', 'max:255'],
            'published_at_date' => ['nullable', 'date'],
            'published_at_time' => ['nullable', 'date_format:H:i'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('post_translations')
                    ->ignore($post->getAttribute('id'), 'post_id')
                    ->where('locale', $locale)
            ],
            'author_id' => ['required', 'integer', 'min:1', 'exists:users,id'],
            'category_id' => ['required', 'integer', 'min:1', 'exists:categories,id'],
            'tags_list' => ['nullable', 'array'],
            'tags_list.*' => ['required', 'integer', 'min:1', 'exists:tags,id'],
            'featured' => ['nullable', 'boolean']
        ]);

        $post->setAttribute('featured', $validatedData['featured'] ?? false);
        $post->setAttribute('video_link', $validatedData['video_link'] ?? null);
        $post->translateOrNew($locale)->title = $validatedData['title'];
        $post->translateOrNew($locale)->intro = $validatedData['intro'] ?? null;
        $post->translateOrNew($locale)->content = $validatedData['content'];
        $post->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['title']);
        $post->author()->associate($validatedData['author_id']);
        $post->category()->associate($validatedData['category_id']);
        $post->setAttribute('published_at', $publishedAt->get($validatedData['published_at_date'], $validatedData['published_at_time']));
        $post->save();
        $post->tags()->sync($validatedData['tags_list'] ?? []);

        return $this->response
            ->redirectToRoute('algo.posts.index')
            ->with('status', 'Objava uređena!');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Post $post, Storage $storage, Picasso $picasso): RedirectResponse
    {
        // delete all post images before deleting the post.
        foreach ($post->getAttribute('images') as $image) {
            $storage::disk('originals')->delete($image->path);
            $picasso->purge($image->path, 'public');
            $image->delete();
        }

        $post->delete();

        return $this->response
            ->redirectToRoute('algo.posts.index')
            ->with('status', 'Objava obrisana!');
    }
}
