<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Lang;
use Illuminate\Support\Facades\Storage;
use Laravelista\Algo\Models\Image;
use Laravelista\Algo\Models\Post;
use Laravelista\Picasso\Picasso;

class PostImageController extends Controller
{
    public function __construct(
        protected ResponseFactory $response,
        protected Picasso $picasso
    ) {
    }

    /**
     * Display a listing of the resource.
     */
    public function index(Post $post): Response
    {
        $images = Image::where('post_id', $post->getAttribute('id'))->latest()->get();

        return $this->response->view(
            view: 'algo::posts.images.index',
            data: [
                'post' => $post,
                'images' => $images
            ]
        );
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(Post $post): Response
    {
        return $this->response->view(
            view: 'algo::posts.images.create',
            data: ['post' => $post]
        );
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request, Post $post): RedirectResponse
    {
        $request->validate([
            'image' => ['required', 'file', 'mimes:jpeg,jpg,png'],
        ]);

        $image = new Image();
        $image->path = $request->image->store('images', 'originals');
        $image->post()->associate($post->getAttribute('id'));
        $image->save();

        $this->picasso->optimize($image->path, Config::get(key: 'algo.post_dimensions'), 'public');

        return $this->response
            ->redirectToRoute(
                route: 'algo.posts.images.index',
                parameters: $post->getAttribute('id')
            )
            ->with('status', Lang::get('algo::algo.controllers.image.stored'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Post $post, Image $image): Response
    {
        return $this->response->view(
            view: 'algo::posts.images.edit',
            data: [
                'post' => $post,
                'image' => $image
            ]
        );
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Post $post, Image $image, Storage $storage): RedirectResponse
    {
        $request->validate([
            'image' => ['required', 'file', 'mimes:jpeg,jpg,png'],
        ]);

        $storage::disk('originals')->delete($image->path);
        $this->picasso->purge($image->path, 'public');

        $image->path = $request->image->store('images', 'originals');
        $image->save();

        $this->picasso->optimize($image->path, Config::get(key: 'algo.post_dimensions'), 'public');

        return $this->response
            ->redirectToRoute(
                route: 'algo.posts.images.index',
                parameters: $post->getAttribute('id')
            )
            ->with('status', Lang::get('algo::algo.controllers.image.updated'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Post $post, Image $image, Storage $storage): RedirectResponse
    {
        $storage::disk('originals')->delete($image->path);
        $this->picasso->purge($image->path, 'public');

        $image->delete();

        return $this->response
            ->redirectToRoute(
                route: 'algo.posts.images.index',
                parameters: $post
            )
            ->with('status', Lang::get('algo::algo.controllers.image.destroyed'));
    }
}
