<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Lang;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;
use Laravelista\Algo\Helpers\Slug;
use Laravelista\Algo\Models\Tag;
use Laravelista\Picasso\Picasso;

class TagController extends Controller
{
    /**
     * Used in store, edit and destroy methods.
     */
    protected const REDIRECT_TO_ROUTE = 'algo.tags.index';

    protected const VIEW_INDEX = 'algo::tags.index';
    protected const VIEW_CREATE = 'algo::tags.create';
    protected const VIEW_SHOW = 'algo::tags.show';
    protected const VIEW_EDIT = 'algo::tags.edit';

    public function __construct(
        protected ResponseFactory $response
    ) {
    }

    /**
     * Display a listing of the resource.
     */
    public function index(): Response
    {
        $tags = Tag::with(['translations'])->get()->sortBy('name');

        return $this->indexResponse($tags);
    }

    protected function indexResponse(Collection $tags): Response
    {
        return $this->response->view(
            view: static::VIEW_INDEX,
            data: ['tags' => $tags]
        );
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(): Response
    {
        return $this->createResponse();
    }

    protected function createResponse(): Response
    {
        return $this->response->view(view: static::VIEW_CREATE);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request, Slug $slug, Picasso $picasso): RedirectResponse
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('tag_translations')->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $tag = new Tag();
        $tag->translateOrNew($locale)->name = $validatedData['name'];
        $tag->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['name']);
        $tag->translateOrNew($locale)->description = $validatedData['description'] ?? null;
        if ($request->hasFile('image')) {
            $tag->setAttribute('image', $request->image->store('images', 'originals'));
        }
        $tag->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($tag->getAttribute('image'), Config::get(key: 'algo.tag_dimensions'), 'public');
        }

        return $this->storeResponse();
    }

    protected function storeResponse(): RedirectResponse
    {
        return $this->response
            ->redirectToRoute(route: static::REDIRECT_TO_ROUTE)
            ->with('status', Lang::get('algo::algo.controllers.tag.stored'));
    }

    /**
     * Display the specified resource.
     */
    public function show(Tag $tag): Response
    {
        return $this->showResponse($tag);
    }

    protected function showResponse(Tag $tag): Response
    {
        return $this->response->view(
            view: static::VIEW_SHOW,
            data: ['tag' => $tag]
        );
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Tag $tag): Response
    {
        return $this->editResponse($tag);
    }

    protected function editResponse(Tag $tag): Response
    {
        return $this->response->view(
            view: static::VIEW_EDIT,
            data: ['tag' => $tag]
        );
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Tag $tag, Slug $slug, Picasso $picasso, Storage $storage): RedirectResponse
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('tag_translations')
                    ->ignore($tag->getAttribute('id'), 'tag_id')
                    ->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $tag->translateOrNew($locale)->name = $validatedData['name'];
        $tag->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['name']);
        $tag->translateOrNew($locale)->description = $validatedData['description'] ?? null;

        if ($request->hasFile('image') and $tag->hasImage()) {
            $storage::disk('originals')->delete($tag->getAttribute('image'));
            $picasso->purge($tag->getAttribute('image'), 'public');
        }

        if ($request->hasFile('image')) {
            $tag->setAttribute('image', $request->image->store('images', 'originals'));
        }

        $tag->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($tag->getAttribute('image'), Config::get(key: 'algo.tag_dimensions'), 'public');
        }

        return $this->updateResponse();
    }

    protected function updateResponse(): RedirectResponse
    {
        return $this->response
            ->redirectToRoute(route: static::REDIRECT_TO_ROUTE)
            ->with('status', Lang::get('algo::algo.controllers.tag.updated'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Tag $tag, Storage $storage, Picasso $picasso): RedirectResponse
    {
        if ($tag->hasImage()) {
            $storage::disk('originals')->delete($tag->getAttribute('image'));
            $picasso->purge($tag->getAttribute('image'), 'public');
        }

        $tag->delete();

        return $this->destroyResponse();
    }

    protected function destroyResponse(): RedirectResponse
    {
        return $this->response
            ->redirectToRoute(route: static::REDIRECT_TO_ROUTE)
            ->with('status', Lang::get('algo::algo.controllers.tag.destroyed'));
    }
}
