<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Lang;
use Illuminate\Validation\Rule;
use Laravelista\Algo\Helpers\Slug;
use Laravelista\Algo\Helpers\ValidateLocale;
use Laravelista\Algo\Models\Tag;

class TagTranslationController extends Controller
{
    protected const REDIRECT_TO_ROUTE = 'algo.tags.translations.edit';

    protected const VIEW_EDIT = 'algo::tags.translations';

    public function __construct(
        protected ResponseFactory $response,
        protected ValidateLocale $validateLocale
    ) {
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Tag $tag, string $locale): Response
    {
        $this->validateLocale->validate($locale);

        return $this->editResponse($tag, $locale);
    }

    protected function editResponse(Tag $tag, string $locale): Response
    {
        return $this->response->view(
            view: static::VIEW_EDIT,
            data: [
                'tag' => $tag,
                'locale' => $locale
            ]
        );
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Tag $tag, string $locale, Slug $slug): RedirectResponse
    {
        $this->validateLocale->validate($locale);

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('tag_translations')
                    ->ignore($tag->getAttribute('id'), 'tag_id')
                    ->where('locale', $locale)
            ],
            'description' => ['nullable', 'string'],
        ]);

        $tag->translateOrNew($locale)->name = $validatedData['name'];
        $tag->translateOrNew($locale)->slug = $slug->get($validatedData['slug'] ?? null, $validatedData['name']);
        $tag->translateOrNew($locale)->description = $validatedData['description'] ?? null;
        $tag->save();
        
        return $this->updateResponse($tag->getAttribute('id'), $locale);
    }

    protected function updateResponse(int $id, string $locale): RedirectResponse
    {
        return $this->response
            ->redirectToRoute(
                route: static::REDIRECT_TO_ROUTE,
                parameters: [
                    $id, $locale
                ]
            )
            ->with('status', Lang::get('algo::algo.controllers.tag_translation.translated'));
    }
}
