<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;
use Laravelista\Algo\Helpers\Slug;
use Laravelista\Picasso\Picasso;

class UserController extends Controller
{
    public function __construct(
        protected ResponseFactory $response
    ) {
    }

    /**
     * Display a listing of the resource.
     */
    public function index(): Response
    {
        $users = Config::get('algo.user')::orderBy('name')->get();

        return $this->response->view(
            view: 'algo::users.index',
            data: ['users' => $users]
        );
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(): Response
    {
        return $this->response->view(view: 'algo::users.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request, Picasso $picasso, Slug $slug): RedirectResponse
    {
        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8', 'confirmed'],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'slug' => ['nullable', 'string', 'max:255', 'unique'],
            'location' => ['nullable', 'string', 'max:255'],
            'website' => ['nullable', 'string', 'max:255', 'url'],
            'facebook_profile' => ['nullable', 'string', 'max:255', 'url'],
            'twitter_profile' => ['nullable', 'string', 'max:255', 'url'],
            'bio' => ['nullable', 'string'],
        ]);

        $user = Config::get('algo.user')::create([
            'name' => $validatedData['name'],
            'email' => $validatedData['email'],
            'password' => Hash::make($validatedData['password'])
        ]);
        $user->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $user->location = $validatedData['location'];
        $user->website = $validatedData['website'];
        $user->facebook_profile = $validatedData['facebook_profile'];
        $user->twitter_profile = $validatedData['twitter_profile'];
        $user->bio = $validatedData['bio'];

        if ($request->hasFile('image')) {
            $user->image = $request->image->store('images', 'originals');
        }

        $user->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($user->image, Config::get(key: 'algo.user_dimensions'), 'public');
        }

        return $this->response
            ->redirectToRoute(route: 'algo.users.index')
            ->with('status', 'Korisnik napravljen!');
    }

    /**
     * Display the specified resource.
     */
    public function show(mixed $user): Response
    {
        $user = Config::get('algo.user')::findOrFail($user);

        return $this->response->view(
            view: 'algo::users.show',
            data: ['user' => $user]
        );
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(mixed $user): Response
    {
        $user = Config::get('algo.user')::findOrFail($user);

        return $this->response->view(
            view: 'algo::users.edit',
            data: ['user' => $user]
        );
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, mixed $user, Picasso $picasso, Storage $storage, Slug $slug): RedirectResponse
    {
        $user = Config::get('algo.user')::findOrFail($user);

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => [
                'required', 'string', 'email', 'max:255',
                Rule::unique(Config::get('algo.user'))->ignore($user->id)
            ],
            'password' => ['nullable', 'string', 'min:8', 'confirmed'],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique(Config::get('algo.user'))->ignore($user->id)
            ],
            'location' => ['nullable', 'string', 'max:255'],
            'website' => ['nullable', 'string', 'max:255', 'url'],
            'facebook_profile' => ['nullable', 'string', 'max:255', 'url'],
            'twitter_profile' => ['nullable', 'string', 'max:255', 'url'],
            'bio' => ['nullable', 'string'],
        ]);

        $user->fill([
            'name' => $validatedData['name'],
            'email' => $validatedData['email'],
        ]);
        $user->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $user->location = $validatedData['location'];
        $user->website = $validatedData['website'];
        $user->facebook_profile = $validatedData['facebook_profile'];
        $user->twitter_profile = $validatedData['twitter_profile'];
        $user->bio = $validatedData['bio'];

        if (!is_null($validatedData['password'])) {
            $user->password = Hash::make($validatedData['password']);
        }

        if ($request->hasFile('image') and $user->hasImage()) {
            $storage::disk('originals')->delete($user->image);
            $picasso->purge($user->image, 'public');
        }

        if ($request->hasFile('image')) {
            $user->image = $request->image->store('images', 'originals');
        }

        $user->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($user->image, Config::get(key: 'algo.user_dimensions'), 'public');
        }

        return $this->response
            ->redirectToRoute(route: 'algo.users.index')
            ->with('status', 'Korisnik uređen!');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(mixed $user, Storage $storage, Picasso $picasso): RedirectResponse
    {
        $user = Config::get('algo.user')::findOrFail($user);

        if ($user->hasImage()) {
            $storage::disk('originals')->delete($user->image);
            $picasso->purge($user->image, 'public');
        }

        $user->delete();

        return $this->response
            ->redirectToRoute(route: 'algo.users.index')
            ->with('status', 'Korisnik obrisan!');
    }
}
