<?php

namespace Laravelista\Algo\Models;

use Astrotomic\Translatable\Translatable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Str;

class Post extends Model
{
    use Translatable;

    /**
     * The attributes that are mass assignable.
     *
     * @var string[]
     */
    protected $fillable = ['published_at', 'content', 'intro', 'title', 'video_link', 'slug', 'featured'];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'featured' => 'boolean'
    ];

    /**
     * The attributes that are translatable.
     */
    public array $translatedAttributes = ['content', 'intro', 'title', 'slug'];

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = ['published_at'];

    /**
     * It returns the author of the post.
     */
    public function author(): BelongsTo
    {
        return $this->belongsTo(Config::get('algo.user'));
    }

    /**
     * It returns the tags for this post.
     */
    public function tags(): BelongsToMany
    {
        return $this->belongsToMany(Tag::class);
    }

    /**
     * It returns the category to which this post belongs to.
     */
    public function category(): BelongsTo
    {
        return $this->belongsTo(Category::class);
    }

    /**
     * It returns images for this post.
     */
    public function images(): HasMany
    {
        return $this->hasMany(Image::class);
    }

    /**
     * It returns the excerpt or content limited to chars.
     */
    public function getExcerpt(int $chars = 200): string
    {
        return $this->getAttribute('intro') ?? Str::limit(strip_tags((new \Parsedown())->parse($this->getAttribute('content'))), $chars);
    }

    /**
     * It converts content from markdown to html
     * and returns it as text.
     */
    public function getContent(): string
    {
        return (new \Parsedown())->parse($this->getAttribute('content'));
    }

    /**
     * It returns the reading time in minutes.
     */
    public function getReadingTime(int $wordsPerMinute = 200): int
    {
        $words = str_word_count($this->getAttribute('content'));

        return (int) ceil((int) $words / $wordsPerMinute);
    }

    /**
     * It returns only currently published posts, not scheduled posts.
     */
    public function scopePublished(Builder $query): Builder
    {
        return $query->whereNotNull('published_at')
            ->where('published_at', '<=', Carbon::now());
    }

    /**
     * It only check if the published_at is null or
     * if the published_at is less than or equals to now.
     */
    public function isPublished(): bool
    {
        if (is_null($this->getAttribute('published_at'))) {
            return false;
        }

        if ($this->getAttribute('published_at')->lte(Carbon::now())) {
            return true;
        }

        return false;
    }

    /**
     * It returns true if the post is featured,
     * or false if it is not featured.
     */
    public function isFeatured(): bool
    {
        return $this->getAttribute('featured');
    }

    /**
     * It returns only posts that are featured.
     */
    public function scopeFeatured(Builder $query): Builder
    {
        return $query->where('featured', true);
    }

    /**
     * It returns the video link in correct youtube embed url.
     */
    public function getYouTubeEmbedLink(): string
    {
        return str_replace('watch?v=', 'embed/', $this->getAttribute('video_link'));
    }

    /**
     * Depending on the post attributes it returns
     * the correct post format to display it in.
     */
    public function getPostFormat(): string
    {
        if (!empty($this->getAttribute('video_link')) and $this->getAttribute('images')->count() > 0) {
            return 'video';
        }

        if ($this->getAttribute('images')->count() > 1) {
            return 'gallery';
        }

        if ($this->getAttribute('images')->count() == 1) {
            return 'image';
        }

        return 'no-image';
    }
}
