<?php

namespace Laravelista\Algo;

use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\ServiceProvider as LaravelServiceProvider;

class ServiceProvider extends LaravelServiceProvider
{
    public function boot(): void
    {
        $this->bootRoutes();

        $this->bootMigrations();

        $this->bootViews();

        $this->bootBladeComponents();

        $this->bootTranslations();

        $this->publishes(
            paths: [
                __DIR__ . '/../config/algo.php' => $this->app->configPath(path: 'algo.php'),
            ],
            groups: 'config'
        );
    }

    public function register(): void
    {
        $this->mergeConfigFrom(
            path: __DIR__ . '/../config/algo.php',
            key: 'algo'
        );

        /**
         * If the translatable config is not published,
         * use values from this package config file.
         *
         * Otherwise, use the values from the translatable
         * config file.
         */
        if (!File::exists(path: $this->app->configPath(path: 'translatable.php'))) {
            Config::set(key: [
                'translatable.locales' => Config::get(key: 'algo.translatable.locales'),
                'translatable.use_fallback' => Config::get(
                    key: 'algo.translatable.use_fallback',
                    default: true
                ),
                'translatable.fallback_locale' => Config::get(
                    key: 'algo.translatable.fallback_locale',
                    default: 'en'
                )
            ]);
        }

        /**
         * Merge picasso dimensions
         *
         * If the picasso config is published, merge this package
         * picasso dimensions with picasso package config dimensions.
         *
         * Else replace picasso config values with this package config values.
         */
        if (File::exists($this->app->configPath(path: 'picasso.php'))) {
            Config::set(key: [
                'picasso.dimensions' => array_merge(
                    Config::get('algo.picasso.dimensions'),
                    Config::get('picasso.dimensions')
                )
            ]);
        } else {
            Config::set(key: [
                'picasso.dimensions' => Config::get(key: 'algo.picasso.dimensions')
            ]);
        }
    }

    protected function bootRoutes(): void
    {
        if (Config::get(
            key: 'algo.ui',
            default: true
        )) {
            $this->loadRoutesFrom(path: __DIR__ . '/routes.php');
        }
    }

    protected function bootMigrations(): void
    {
        $this->loadMigrationsFrom(paths: __DIR__ . '/../migrations');

        $this->publishes(
            paths: [
                __DIR__.'/../migrations/' => App::databasePath(path: 'migrations')
            ],
            groups: 'migrations'
        );
    }

    protected function bootViews(): void
    {
        $this->loadViewsFrom(
            path: __DIR__ . '/../resources/views',
            namespace: 'algo'
        );

        $this->publishes(
            paths: [
                __DIR__ . '/../resources/views' => App::resourcePath(path: 'views/vendor/algo'),
            ],
            groups: 'views'
        );
    }

    protected function bootBladeComponents(): void
    {
        Blade::include(
            path: 'algo::ui.bootstrap4.forms.input',
            alias: 'input'
        );
        Blade::include(
            path: 'algo::ui.bootstrap4.forms.checkbox',
            alias: 'checkbox'
        );
        Blade::include(
            path: 'algo::ui.bootstrap4.forms.textarea',
            alias: 'textarea'
        );
        Blade::include(
            path: 'algo::ui.bootstrap4.forms.select',
            alias: 'select'
        );
        Blade::include(
            path: 'algo::ui.bootstrap4.forms.submit',
            alias: 'submit'
        );
        Blade::aliasComponent(
            path: 'algo::ui.bootstrap4.forms.form',
            alias: 'form'
        );

        Blade::include(
            path: 'algo::ui.bootstrap4.edit.language_switcher',
            alias: 'languageSwitcher'
        );

        Blade::include(
            path: 'algo::ui.bootstrap4.show.field',
            alias: 'field'
        );
        Blade::include(
            path: 'algo::ui.bootstrap4.show.tablist',
            alias: 'tablist'
        );
        Blade::aliasComponent(
            path: 'algo::ui.bootstrap4.show.tabpanel',
            alias: 'tabpanel'
        );

        Blade::include(
            path: 'algo::ui.bootstrap4.status_alert',
            alias:'statusAlert'
        );

        Blade::aliasComponent(
            path: 'algo::ui.bootstrap4.dropdowns.split_button',
            alias:'dropdownSplitButton'
        );
    }

    protected function bootTranslations(): void
    {
        $this->loadTranslationsFrom(
            path: __DIR__ . '/../resources/lang',
            namespace: 'algo'
        );

        $this->publishes(
            paths: [
                __DIR__ . '/../resources/lang' => App::resourcePath(path: 'lang/vendor/algo'),
            ],
            groups: 'translations'
        );
    }
}
